#import "VTController.h"
#import "VTSearcher.h"

@implementation VTController

- (id)init
{
	[super init];
	searcher = [[VTSearcher alloc] initWithDelegate:self];
	tripArr = [[NSMutableArray alloc] init];
	keyArr = [[NSMutableArray alloc] init];
	isRunning = NO;
	return self;
}

- (IBAction)beginSearch:(id)sender
{
	BOOL flag,flag2,flag3;
	if(!isRunning) {
		if([[o_searchType selectedCell] tag] == 0) {
			if([[o_searchMode selectedItem] tag] < 3 && [[o_searchStr stringValue] isEqualToString: @""]) {
				NSBeginCriticalAlertSheet(NSLocalizedString(@"error", nil), @"OK", nil, nil, o_mainWindow, nil, nil, nil, nil, NSLocalizedString(@"null string", nil));
				return;
			}
			if([o_caseRadio state] == NSOnState) flag = YES;
			else flag = NO;
			if([o_outputFile state] == NSOnState) flag2 = YES;
			else flag2 = NO;
			if([o_studyRE state] == NSOnState) flag3 = YES;
			else flag3 = NO;
			[searcher beginSearch:[o_searchStr stringValue] mode:[[o_searchMode selectedItem] tag] ignoreCase:flag minLength:[o_minRepeat intValue] log:flag2 study:flag3];
		}
		else {
			if([o_outputFile state] == NSOnState) flag = YES;
			else flag = NO;
			//[searcher beginPatternSearch:flag method:[[o_matchMethod selectedItem] tag]];
			[searcher beginPatternSearch:flag method:0];
		}
		[o_speedStr setStringValue:@""];
		[o_button setTitle:NSLocalizedString(@"stopsearch", nil)];
		[o_clearMenu setEnabled:NO];
		isRunning = YES;
		return;
	}
	[searcher stopSearch];
	[o_button setTitle:NSLocalizedString(@"beginsearch", nil)];
	[o_clearMenu setEnabled:YES];
	isRunning = NO;
}

- (IBAction)showPref:(id)sender
{
	[o_prefPane center];
	[o_prefPane makeKeyAndOrderFront:self];
}

- (IBAction)prefOK:(id)sender
{
	[o_prefPane close];
}

- (IBAction)clearTable:(id)sender
{
	[tripArr removeAllObjects];
	[keyArr removeAllObjects];
	[o_tableView reloadData];
}

- (IBAction)setLogPath:(id)sender
{
	NSOpenPanel *op;
	int stat;
	
	op = [NSOpenPanel openPanel];
	[op setTitle: NSLocalizedString(@"log path",@"")];
	[op setCanChooseDirectories: YES];
	[op setCanChooseFiles: NO];
	[op setAllowsMultipleSelection: NO];
	stat = [op runModalForDirectory: nil file: nil types: nil];
							   
	if(stat != NSOKButton) return;
	[o_logPath setStringValue:[op filename]];
}

- (void)tripFound:(char *)trip key:(char *)key
{
	[tripArr addObject: [NSString stringWithFormat: @"%@%s",NSLocalizedString(@"daia", nil),trip]];
	[keyArr addObject: [NSString stringWithFormat: @"#%s",key]];
	[o_tableView reloadData];
	[o_tableView scrollRowToVisible: [tripArr count]-1];
}

- (void)setCurrentSpeed:(double)speed
{
	[o_speedStr setDoubleValue:speed];
}

- (NSString *)logPath
{
	return [o_logPath stringValue];
}

- (void)error:(char *)str type:(int)t
{
	switch(t) {
		case 0:
			NSBeginCriticalAlertSheet(NSLocalizedString(@"regex error", nil), @"OK", nil, nil, o_mainWindow, nil, nil, nil, nil, [NSString stringWithCString:str]);
			break;
		case 1:
			NSBeginCriticalAlertSheet(NSLocalizedString(@"log error", nil), @"OK", nil, nil, o_mainWindow, nil, nil, nil, nil, NSLocalizedString(@"can't write log", nil));
			break;
		case 2:
			NSBeginCriticalAlertSheet(NSLocalizedString(@"pattern error", nil), @"OK", nil, nil, o_mainWindow, nil, nil, nil, nil, NSLocalizedString(@"can't read pattern", nil));
			break;
		case 3:
			NSBeginCriticalAlertSheet(NSLocalizedString(@"pattern error", nil), @"OK", nil, nil, o_mainWindow, nil, nil, nil, nil, NSLocalizedString(@"pattern file is empty", nil));
			break;
		default:
			break;
	}
	[self beginSearch:nil];
}

/* prefs */

- (void)savePrefs
{
	NSUserDefaults *pref = [NSUserDefaults standardUserDefaults];
	[pref setInteger: [o_caseRadio state] forKey: @"IsCaseInsensitive"];
	[pref setObject:[o_logPath stringValue] forKey:@"OutputDir"];
	[pref setInteger: [o_outputFile state] forKey: @"ShouldLog"];
	[pref setInteger:[[o_clipboardStyle selectedCell] tag] forKey:@"ClipboardStyle"];
	[pref setInteger: [o_studyRE state] forKey: @"StudyRE"];
	[pref setInteger:[[o_searchMode selectedItem] tag] forKey:@"SearchMode"];
	[pref setInteger: [o_minRepeat intValue] forKey : @"MinRepeat"];
	[pref setInteger:[[o_searchType selectedCell] tag] forKey:@"SearchType"];
	//[pref setInteger:[[o_matchMethod selectedItem] tag] forKey:@"MatchMethod"];
	[pref synchronize];
}

- (void)loadPrefs
{
	NSUserDefaults *pref = [NSUserDefaults standardUserDefaults];
	id temp;
	int i;
	if(temp=[pref objectForKey:@"ClipboardStyle"]) {
		i = [temp intValue];
		[o_clipboardStyle selectCellWithTag: i];
	}
	if(temp=[pref objectForKey:@"OutputDir"]) {
		[o_logPath setStringValue:temp];
	}
	else [o_logPath setStringValue:[@"~/Desktop" stringByExpandingTildeInPath]];
	if(temp=[pref objectForKey:@"IsCaseInsensitive"]) {
		[o_caseRadio setState: [temp intValue]];
	}
	if(temp=[pref objectForKey:@"ShouldLog"]) {
		[o_outputFile setState: [temp intValue]];
	}
	if(temp=[pref objectForKey:@"StudyRE"]) {
		[o_studyRE setState: [temp intValue]];
	}
	if(temp=[pref objectForKey:@"SearchMode"]) {
		[o_searchMode selectItemAtIndex: [o_searchMode indexOfItemWithTag:[temp intValue]]];
	}
	if(temp=[pref objectForKey:@"MinRepeat"]) {
		[o_minRepeat setIntValue: [temp intValue]];
		[o_minRepeatStepper setIntValue: [temp intValue]];
	}
	if(temp=[pref objectForKey:@"SearchType"]) {
		i = [temp intValue];
		[o_searchType selectCellWithTag: i];
	}
	/*if(temp=[pref objectForKey:@"MatchMethod"]) {
		[o_matchMethod selectItemAtIndex: [o_searchMode indexOfItemWithTag:[temp intValue]]];
	}*/
}


/* delegate methods */

- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
	return [tripArr count];
}

- (id)tableView:(NSTableView *)tableView
		objectValueForTableColumn:(NSTableColumn *)tableColumn
		row:(int)row
{
	id identifier = [tableColumn identifier];
	
	if([identifier isEqual:@"trip"])  {
		return [tripArr objectAtIndex: row];
	}
	else if([identifier isEqual:@"key"]){
		return [keyArr objectAtIndex: row];
	}
	return nil;
}

- (void)applicationDidFinishLaunching: (NSNotification *)notification
{
	[self loadPrefs];
	[o_tableView setDoubleAction:@selector(copy:)];
	[o_mainWindow makeKeyAndOrderFront: self];
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
	if(isRunning) {
		int ret = NSRunAlertPanel(NSLocalizedString(@"Quit",nil), NSLocalizedString(@"Quit anyway?",nil),@"OK", nil, @"Cancel");
		if(ret == NSAlertOtherReturn) return NO;
	}
	[self savePrefs];
	return YES;
}

- (BOOL)windowShouldClose:(id)sender
{
	[NSApp terminate:nil];
	return NO;
}

- (IBAction)copy:(id)sender
{
	int row = [o_tableView selectedRow];
	if(row < 0) return;
	NSPasteboard *pb = [NSPasteboard generalPasteboard];
	NSMutableString *str = [NSMutableString string];
	[pb declareTypes: [NSArray arrayWithObjects :NSStringPboardType, nil] owner: self];
	int n = [keyArr count];
	int i;
	BOOL first = YES;
	if([[o_clipboardStyle selectedCell]tag] == 0) {
		for(i=0;i<n;i++) {
			if([o_tableView isRowSelected:i]) {
				if(first) { [str appendString:[keyArr objectAtIndex:i]]; first = NO; }
				else [str appendString:[NSString stringWithFormat:@"\n%@",[keyArr objectAtIndex:i]]];
			}
		}
	}
	else {
		for(i=0;i<n;i++) {
			if([o_tableView isRowSelected:i]) {
				if(first) { [str appendString:[NSString stringWithFormat: @"%@ : %@",[tripArr objectAtIndex:i], [keyArr objectAtIndex:i]]]; first = NO; }
				else [str appendString:[NSString stringWithFormat: @"\n%@ : %@",[tripArr objectAtIndex:i], [keyArr objectAtIndex:i]]];
			}
		}
	}
	[pb setString: str forType: NSStringPboardType];
}

@end
